#!/usr/bin/perl -w
use strict;
# ----------------------------------------------------------------------
#
# calendar.cgi
#    --	Display a calendar of events for any month between
#	January 1970 and December 2049.
#
# Copyright Collin Forbes 2000.  All rights reserved.
# This program is free software; you can redistribute it and/or modify
# it under the same terms as Perl itself.
#
# ----------------------------------------------------------------------
# ----------------------------------------------------------------------
#
# Summary
# -------
#   A program to show a calendar of events. Make calendars for months
#   between Jan, 1970 and Dec, 2049 using customizable templates and a
#   stylesheet.
#
# Version History
# ---------------
#   * Version 2.3.3, released 20 April 2000
#   * Version 2.3.2, released February 1999
#   * Version 2.3.1, released January 1999
#   * Version 2.3, released February 1998
#   * Version 2.2, released August 1996
#   * ...and a few earlier versions lost in the shuffle of time.
#
# See Also
# --------
#   * http://kuoi.asui.uidaho.edu/~collinf/scripts.html/calendar
#
# ----------------------------------------------------------------------

my $default_file = '/home/collinf/public_html/programs/calendar/config.txt';
#
# $default_file is the path (either absolute or relative) to the 
# configuration file # you want to use when you didn't ask for an 
# explicit file as part of the URL.
#
# ----------------------------------------------------------------------
#
# CGI::Carp is a standard module.  Complain to your system administator 
# if you don't have it.  The other modules can either be installed by
# your system adminstrator (where everybody can use them), or simply
# copied into one of your own directories.
# 
# Uncomment and change this to the directory where you put the modules.
# use lib 'custom/library/path';
  
use CGI::Carp qw(fatalsToBrowser);
use CGI::Minimal;
use Template;

my $header_sent;

# ----------------------------------------------------------------------

my $url = $ENV{'SCRIPT_NAME'} || 'calendar.cgi';
#
# $url is the URL of the calendar script, used to build links
# back to other calendars (next and previous) from the template.
#

my @month_names = ( 
    'null', 'january', 'february', 'march', 'april', 'may', 'june',
    'july', 'august', 'september', 'october', 'november', 'december'
);

my @days_in_month = ( 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31,);
#
# @month_names stores the names of the months.
# @days_in_month keeps track of the days in each month.
#
# Thirty-one days hath Jan, March, May, July, Aug, Oct, and Dec.
# Leap years are handled in the perpetual_calendar subroutine.
#
# the 'null' and 0 values at the beginning of each list are placeholders
# to let the month names run from 1-12 rather than 0-11.
#
# ----------------------------------------------------------------------
#
# figure out what calendar to show -- the filename and the year/month
#
my $filename = $ENV{'PATH_TRANSLATED'} || $default_file || 'urls.txt';
$filename =~ s/[^\w\/\-\.]/_/g;	# lingering paranoia

my $cgi = CGI::Minimal->new;
my( $year, $month ) = sensible_dates( $cgi->param('year') || 'current',
				      $cgi->param('month') || 'current' );

#
# read the configuration file and parse the relevant data
#
my $data = read_configuration_file( $filename );
my( $directives, $events ) = parse_configuration( $data, $year, $month );

#
# prepare the template we're going to use
#
my $tmpl = Template->initialize( $directives->{'template'} || 'calendar.tmpl' );
confess( qq|Template: | . $tmpl->errstr ) if $tmpl->errstr;

#
# make the calendar table or a lynx-friendly list
#
my $table;
if ( $ENV{'HTTP_USER_AGENT'} and $ENV{'HTTP_USER_AGENT'} =~ /lynx/i ) { 
    $table = calendar_list( $events, $year, $month );
}
else { $table = calendar_table( $directives, $events, $year, $month ); }
$tmpl->token( 'calendar', $table );

#
# add the URLs to navigate to the previous and next months
#
my $prev_href = build_previous_month_href( $url, $year, $month );
my $next_href = build_next_month_href( $url, $year, $month );
$tmpl->token( 'prev', $prev_href );
$tmpl->token( 'next', $next_href );

#
# now spit out the response and be done
#
print_response( $tmpl->merge );
exit;

# ----------------------------------------------------------------------
# ----------------------------------------------------------------------

sub calendar_table {
#
# Prints the calendar for the given month and year as a HTML table.
# The heading and table cells might be specially styled if the 
# configuration directives allow.
#
    my( $directives, $events, $year, $month ) = @_;

    #
    # Let's figure out the table decorations.  Stylesheets are preferred,
    # but background colors will be tolerated.
    #
    my $head_attribute = 'CLASS="heading"';
    if ( $directives->{'head_bgcolor'} ) { 
	$head_attribute = qq|BGCOLOR="$directives->{'head_bgcolor'}"|;
    }

    my $event_attribute = 'CLASS="event"';
    if ( $directives->{'event_bgcolor'} ) {
	$event_attribute = qq|BGCOLOR="$directives->{'event_bgcolor'}"|;
    }

    my $cell_attribute = 'CLASS="cell"';
    if ( $directives->{'cell_bgcolor'} ) {
	$cell_attribute = qq|BGCOLOR="$directives->{'cell_bgcolor'}"|;
    }

    # find the cellpadding and cellspacing, if desired
    my $cellpadding = '';
    my $cellspacing = '';
    if ( $directives->{'cellpadding'} ) { 
	$cellpadding = qq|CELLPADDING="$directives->{'cellpadding'}"|;
    }
    if ( $directives->{'cellspacing'} ) {
	$cellspacing = qq|CELLSPACING="$directives->{'cellspacing'}"|;
    }

    # find the table and cell sizes if desired
    my $table_width = '';
    my $cell_width = '';
    if ( $directives->{'table_width'} ) {
	my $width = $directives->{'table_width'};
	if ( $width =~ /^\d+$/ ) { 
	    $cell_width = qq|WIDTH="| . int($width/7) . qq|"|;
	}
	else { $cell_width = 'WIDTH="15%"'; }
	$table_width = qq|WIDTH="$width"|;
    }

    my $cell_height = '';
    if ( $directives->{'cell_height'} ) {
	$cell_height .= qq|HEIGHT="$directives->{'cell_height'}"|;
    }


    # put together the heading of the table
    my $table = <<"EOH";
<TABLE $table_width $cellpadding $cellspacing BORDER="1">
<TR VALIGN="top">
<TH $head_attribute $table_width COLSPAN="7">
<B>\u$month_names[$month] $year</B></TH></TR>
EOH

    # the cell headings with the days of the week
    $table .= <<"EOH";
<TR VALIGN=top>
<TH $head_attribute $cell_width>Sunday</TH>
<TH $head_attribute $cell_width>Monday</TH>
<TH $head_attribute $cell_width>Tuesday</TH>
<TH $head_attribute $cell_width>Wednesday</TH>
<TH $head_attribute $cell_width>Thursday</TH>
<TH $head_attribute $cell_width>Friday</TH>
<TH $head_attribute $cell_width>Saturday</TH></TR>
EOH

    #
    # the space before the first day of the table
    #
    my $first_day = perpetual_calendar( $year, $month );
    if ( $first_day > 0 ) {

	$table .= <<"EOH";
<TD $cell_attribute $cell_height COLSPAN="$first_day">&nbsp;</TD>
EOH

    }

    #
    # show the "meat" of table
    #
    my $week_count = $first_day;
    my $days_in_this_month = $days_in_month[$month];
    if ( $month == 2  and leap_year( $year ) ) { $days_in_this_month = 29; }
    for ( my $count = 1; $count <= $days_in_this_month; $count++ ) {
	my $day_code = join( '.', $year, $month, $count );

	my $attribute = $cell_attribute;
	$attribute = $event_attribute if $events->{ $day_code };

	my $cell_contents = $events->{ $day_code } || '';
        $table .= <<"EOH";
<TD $attribute $cell_width $cell_height VALIGN="top">
<B>$count</B><BR>
$cell_contents</TD>
EOH

	$week_count++;
	if ( $week_count == 7 and $count < $days_in_this_month) { 
	    $table .= qq|</TR>\n<TR VALIGN="top">\n|;
	    $week_count = 0;
	}
    }

    #
    # add the remainder cells -- "&nbsp;" keeps the cell from collapsing
    #
    if ( $week_count < 7 ) {
        my $days_left = 7 - $week_count;

        $table .= <<"EOH";
<TD $cell_attribute COLSPAN="$days_left">&nbsp;</TD></TR>
EOH

    }

    #
    # put monthly notices (those with 'any') into a row at the end
    #
    if ( $events->{ join( '.', $year, $month, 'any' ) } ) {

	$table .= <<"EOH";
<TR VALIGN="top" $event_attribute $cell_height>
<TD $table_width $event_attribute COLSPAN="7">
$events->{ join('.', $year, $month, 'any') }</TD></TR>
EOH

    }

    $table .= qq|</TABLE>\n|;
    return $table;
}

# ----------------------------------------------------------------------

sub calendar_list {
#
# Prints the calendar in order list (OL) form.  For compatibility 
# with Lynx browsers.  It's very simple and not very burdensome. 
# The table is still sexier, using style sheets and such.
#
    my( $events, $year, $month ) = @_;

    my $list = qq|<H2 ALIGN=left>\u$month_names[$month] $year</H2>\n|;

    #
    # Here, monthly events are just put at the top of the calendar.  Simpler.
    #
    if ( $events->{ join( '.', $year, $month, 'any' ) } ) {
	$list .= qq|<P>$events->{ join('.', $year, $month, 'any' ) }\n|;
    }

    #
    # build the ordered list of events
    #
    $list .= qq|<OL>\n|;
    my $days_in_this_month = $days_in_month[$month];
    if ( $month == 2 and leap_year( $year ) ) { $days_in_this_month = 29; }
    for ( my $count = 1; $count <= $days_in_this_month; $count++ ) {
	my $day_code = join( '.', $year, $month, $count );
	if ($events->{$day_code}) { $list .= qq|<LI> $events->{$day_code}\n|; }
	else { $list .= qq|<LI> &nbsp;\n|; }
    }
    $list .= qq|</OL>\n|; 

    return $list;
}

# ----------------------------------------------------------------------

sub perpetual_calendar {
#
# This perpetual calendar was detailed on a wallet-sized card published
# by Arthur A. Merrill and Popular Science in 1952.
#
# This subroutine takes the month and year as arguments and returns what
# "type" of month it is [0-6].  Not coincidently, this is also the day of
# the week the month starts on, where Sunday is a "0" and Saturday is a "6".
#
    my( $year, $month ) = @_;

    #
    # Year codes provided for 1970-2049.  The codes repeat every 150 years.
    # The unix system clock will roll over sometime in 2038.
    #
    my %year_code = (
	'1970'=>'D', '1971'=>'E', '1972'=>'M', '1973'=>'A', '1974'=>'B', 
	'1975'=>'C', '1976'=>'K', '1977'=>'F', '1978'=>'G', '1979'=>'A',

	'1980'=>'I', '1981'=>'D', '1982'=>'E', '1983'=>'F', '1984'=>'N',
	'1985'=>'B', '1986'=>'C', '1987'=>'D', '1988'=>'L', '1989'=>'G',
 
	'1990'=>'A', '1991'=>'B', '1992'=>'J', '1993'=>'E', '1994'=>'F',
	'1995'=>'G', '1996'=>'H', '1997'=>'C', '1998'=>'D', '1999'=>'E',

	'2000'=>'M', '2001'=>'A', '2002'=>'B', '2003'=>'C', '2004'=>'K',
	'2005'=>'F', '2006'=>'G', '2007'=>'A', '2008'=>'I', '2009'=>'D',

	'2010'=>'E', '2011'=>'F', '2012'=>'N', '2013'=>'B', '2014'=>'C',
	'2015'=>'D', '2016'=>'L', '2017'=>'G', '2018'=>'A', '2019'=>'B',

	'2020'=>'J', '2021'=>'E', '2022'=>'F', '2023'=>'G', '2024'=>'H',
	'2025'=>'C', '2026'=>'D', '2027'=>'E', '2028'=>'M', '2029'=>'A',

	'2030'=>'B', '2031'=>'C', '2032'=>'K', '2033'=>'F', '2034'=>'G',
	'2035'=>'A', '2036'=>'I', '2037'=>'D', '2038'=>'E', '2039'=>'F',

	'2040'=>'N', '2041'=>'B', '2042'=>'C', '2043'=>'D', '2044'=>'L',
	'2045'=>'G', '2046'=>'A', '2047'=>'B', '2048'=>'J', '2049'=>'E',
    );

    my $calendar = 0;
    if ( $month == 1 ) {
	if ( $year_code{$year} =~ /[EL]/ ) { $calendar = 5 }
	elsif ( $year_code{$year} =~ /[GN]/ ) { $calendar = 0 }
	elsif ( $year_code{$year} =~ /[DK]/ ) { $calendar = 4 }
	elsif ( $year_code{$year} =~ /[AH]/ ) { $calendar = 1 }
	elsif ( $year_code{$year} =~ /[CJ]/ ) { $calendar = 3 }
	elsif ( $year_code{$year} =~ /[BI]/ ) { $calendar = 2 }
	elsif ( $year_code{$year} =~ /[FM]/ ) { $calendar = 6 }
    }
    elsif ( $month == 2 ) {
	if ( $year_code{$year} =~ /[BI]/ ) { $calendar = 5 }
	elsif ( $year_code{$year} =~ /[DK]/ ) { $calendar = 0 }
	elsif ( $year_code{$year} =~ /[AH]/ ) { $calendar = 4 }
	elsif ( $year_code{$year} =~ /[EL]/ ) { $calendar = 1 }
	elsif ( $year_code{$year} =~ /[GN]/ ) { $calendar = 3 }
	elsif ( $year_code{$year} =~ /[FM]/ ) { $calendar = 2 }
	elsif ( $year_code{$year} =~ /[CJ]/ ) { $calendar = 6 }
    }
    elsif ( $month == 3 ) {
	if ( $year_code{$year} =~ /[BH]/ ) { $calendar = 5 }
	elsif ( $year_code{$year} =~ /[DJ]/ ) { $calendar = 0 }
	elsif ( $year_code{$year} =~ /[AN]/ ) { $calendar = 4 }
	elsif ( $year_code{$year} =~ /[EK]/ ) { $calendar = 1 }
	elsif ( $year_code{$year} =~ /[GM]/ ) { $calendar = 3 }
	elsif ( $year_code{$year} =~ /[FL]/ ) { $calendar = 2 }
	elsif ( $year_code{$year} =~ /[CI]/ ) { $calendar = 6 }
    }
    elsif ( $month == 4 ) {
	if ( $year_code{$year} =~ /[FL]/ ) { $calendar = 5 }
	elsif ( $year_code{$year} =~ /[AN]/ ) { $calendar = 0 }
	elsif ( $year_code{$year} =~ /[EK]/ ) { $calendar = 4 }
	elsif ( $year_code{$year} =~ /[BH]/ ) { $calendar = 1 }
	elsif ( $year_code{$year} =~ /[DJ]/ ) { $calendar = 3 }
	elsif ( $year_code{$year} =~ /[CI]/ ) { $calendar = 2 }
	elsif ( $year_code{$year} =~ /[GM]/ ) { $calendar = 6 }
    }
    elsif ( $month == 5 ) {
	if ( $year_code{$year} =~ /[DJ]/ ) { $calendar = 5 }
	elsif ( $year_code{$year} =~ /[FL]/ ) { $calendar = 0 }
	elsif ( $year_code{$year} =~ /[CI]/ ) { $calendar = 4 }
	elsif ( $year_code{$year} =~ /[GM]/ ) { $calendar = 1 }
	elsif ( $year_code{$year} =~ /[BH]/ ) { $calendar = 3 }
	elsif ( $year_code{$year} =~ /[AN]/ ) { $calendar = 2 }
	elsif ( $year_code{$year} =~ /[EK]/ ) { $calendar = 6 }
    }
    elsif ( $month == 6 ) {
	if ( $year_code{$year} =~ /[AN]/ ) { $calendar = 5 }
	elsif ( $year_code{$year} =~ /[CI]/ ) { $calendar = 0 }
	elsif ( $year_code{$year} =~ /[GM]/ ) { $calendar = 4 }
	elsif ( $year_code{$year} =~ /[DJ]/ ) { $calendar = 1 }
	elsif ( $year_code{$year} =~ /[FL]/ ) { $calendar = 3 }
	elsif ( $year_code{$year} =~ /[EK]/ ) { $calendar = 2 }
	elsif ( $year_code{$year} =~ /[BH]/ ) { $calendar = 6 }
    }
    elsif ( $month == 7 ) {
	if ( $year_code{$year} =~ /[FL]/ ) { $calendar = 5 }
	elsif ( $year_code{$year} =~ /[AN]/ ) { $calendar = 0 }
	elsif ( $year_code{$year} =~ /[EK]/ ) { $calendar = 4 }
	elsif ( $year_code{$year} =~ /[BH]/ ) { $calendar = 1 }
	elsif ( $year_code{$year} =~ /[DJ]/ ) { $calendar = 3 }
	elsif ( $year_code{$year} =~ /[CI]/ ) { $calendar = 2 }
	elsif ( $year_code{$year} =~ /[GM]/ ) { $calendar = 6 }
    }
    elsif ( $month == 8 ) {
	if ( $year_code{$year} =~ /[CI]/ ) { $calendar = 5 }
	elsif ( $year_code{$year} =~ /[EK]/ ) { $calendar = 0 }
	elsif ( $year_code{$year} =~ /[BH]/ ) { $calendar = 4 }
	elsif ( $year_code{$year} =~ /[FL]/ ) { $calendar = 1 }
	elsif ( $year_code{$year} =~ /[AN]/ ) { $calendar = 3 }
	elsif ( $year_code{$year} =~ /[GM]/ ) { $calendar = 2 }
	elsif ( $year_code{$year} =~ /[DJ]/ ) { $calendar = 6 }
    }
    elsif ( $month == 9 ) {
	if ( $year_code{$year} =~ /[GM]/ ) { $calendar = 5 }
	elsif ( $year_code{$year} =~ /[BH]/ ) { $calendar = 0 }
	elsif ( $year_code{$year} =~ /[FL]/ ) { $calendar = 4 }
	elsif ( $year_code{$year} =~ /[CI]/ ) { $calendar = 1 }
	elsif ( $year_code{$year} =~ /[EK]/ ) { $calendar = 3 }
	elsif ( $year_code{$year} =~ /[DJ]/ ) { $calendar = 2 }
	elsif ( $year_code{$year} =~ /[AN]/ ) { $calendar = 6 }
    }
    elsif ( $month == 10 ) {
	if ( $year_code{$year} =~ /[EK]/ ) { $calendar = 5 }
	elsif ( $year_code{$year} =~ /[GM]/ ) { $calendar = 0 }
	elsif ( $year_code{$year} =~ /[DJ]/ ) { $calendar = 4 }
	elsif ( $year_code{$year} =~ /[AN]/ ) { $calendar = 1 }
	elsif ( $year_code{$year} =~ /[CI]/ ) { $calendar = 3 }
	elsif ( $year_code{$year} =~ /[BH]/ ) { $calendar = 2 }
	elsif ( $year_code{$year} =~ /[FL]/ ) { $calendar = 6 }
    }
    elsif ( $month == 11 ) {
	if ( $year_code{$year} =~ /[BH]/ ) { $calendar = 5 }
	elsif ( $year_code{$year} =~ /[DJ]/ ) { $calendar = 0 }
	elsif ( $year_code{$year} =~ /[AN]/ ) { $calendar = 4 }
	elsif ( $year_code{$year} =~ /[EK]/ ) { $calendar = 1 }
	elsif ( $year_code{$year} =~ /[GM]/ ) { $calendar = 3 }
	elsif ( $year_code{$year} =~ /[FL]/ ) { $calendar = 2 }
	elsif ( $year_code{$year} =~ /[CI]/ ) { $calendar = 6 }
    }
    elsif ( $month == 12 ) {
	if ( $year_code{$year} =~ /[GM]/ ) { $calendar = 5 }
	elsif ( $year_code{$year} =~ /[BH]/ ) { $calendar = 0 }
	elsif ( $year_code{$year} =~ /[FL]/ ) { $calendar = 4 }
	elsif ( $year_code{$year} =~ /[CI]/ ) { $calendar = 1 }
	elsif ( $year_code{$year} =~ /[EK]/ ) { $calendar = 3 }
	elsif ( $year_code{$year} =~ /[DJ]/ ) { $calendar = 2 }
	elsif ( $year_code{$year} =~ /[AN]/ ) { $calendar = 6 }
    }

    return $calendar;
}

# ----------------------------------------------------------------------

sub leap_year {
#
# Hlere's the leap year kludge.  Instead of figuring out leap years
# mathematically (and trusting my math), it's a lookup table for 
# 1970-2048.
#
    my( $year ) = @_;
    my( %leaps ) = (
	1972 => 1972, 1976 => 1976, 1980 => 1980, 1984 => 1984,
	1988 => 1988, 1992 => 1992, 1996 => 1996, 2000 => 2000,
	2004 => 2004, 2008 => 2008, 2012 => 2012, 2016 => 2016,
	2020 => 2020, 2024 => 2024, 2028 => 2028, 2032 => 2032,
	2036 => 2036, 2040 => 2040, 2044 => 2044, 2048 => 2048,
    );

    return 1 if $leaps{ $year };
    return 0;
}

# ----------------------------------------------------------------------

sub build_next_month_href {
#
# build the href for the "next month" link
#
    my( $url, $year, $month ) = @_;

    my $filename = $ENV{'PATH_INFO'} || '';

    if ( $month == 12 ) { $month = 1; $year += 1; }
    else { $month += 1; }

    return qq|$url$filename?month=$month&amp;year=$year|;
}

# ----------------------------------------------------------------------

sub build_previous_month_href { 
#
# build the href for the "previous month" link
#
    my( $url, $year, $month ) = @_;

    my $filename = $ENV{'PATH_INFO'} || '';

    if ( $month == 1 ) { $month = 12; $year -= 1; }
    else { $month -= 1; }


    return qq|$url$filename?month=$month&amp;year=$year|;
}

# ----------------------------------------------------------------------

sub sensible_dates {
#
# returns sensible values for the year, month, and day
# the 'sensible values' are based on the current time, plus modifiers 
# to the year and month.
#
    my( $mod_year, $mod_month ) = @_;

    #
    # verify and adjust the month according to the desired month or modifier
    #
    my $month = 1 + (localtime(time))[4];
    if    ( $mod_month =~ /jan/i ) { $month = 1; }
    elsif ( $mod_month =~ /feb/i ) { $month = 2; }
    elsif ( $mod_month =~ /mar/i ) { $month = 3; }
    elsif ( $mod_month =~ /apr/i ) { $month = 4; }
    elsif ( $mod_month =~ /may/i ) { $month = 5; }
    elsif ( $mod_month =~ /jun/i ) { $month = 6; }
    elsif ( $mod_month =~ /jul/i ) { $month = 7; }
    elsif ( $mod_month =~ /aug/i ) { $month = 8; }
    elsif ( $mod_month =~ /sep/i ) { $month = 9; }
    elsif ( $mod_month =~ /oct/i ) { $month = 10; }
    elsif ( $mod_month =~ /nov/i ) { $month = 11; }
    elsif ( $mod_month =~ /dec/i ) { $month = 12; }
    if ( $mod_month =~ /next/i ) {
	if ( $month == 12 ) { $month = 1; $year += 1; }
	else { $month += 1; }
    }
    elsif ( $mod_month =~ /prev/i ) { 
	if ( $month == 1 ) { $month = 12; $year -= 1; }
	else { $month -= 1; }
    }
    elsif ( $mod_month =~ /^\d+$/ ) { 
	if (($mod_month > 0) and ($mod_month < 13)) { $month = $mod_month; }
    }

    #
    # verify and adjust the year according to the modifiers
    #
    my $year = 1900 + (localtime(time))[5];
    if ( $mod_year =~ /next/i ) { $year += 1; }
    elsif ( $mod_year =~ /prev/i ) { $year -= 1; }
    elsif ( $mod_year =~ /^\d{4}$/ ) { $year = $mod_year; }

    return( $year, $month );
}

# ----------------------------------------------------------------------

sub read_configuration_file {
#
# reads from the specified filename and returns a list reference
# lines beginning with # and ; are ignored as comments
#
    my $filename = shift;

    open(FILE, "<$filename" ) 
			or confess( qq|couldn't open file "$filename"| );

    my @data;
    while( <FILE> ) { 
	next if $_ =~ /^#/ or $_ =~ /^;/;	# skip comment lines
	chomp;					# strip newlines
	push( @data, $_ );
    }

    close(FILE);

    return \@data;
}

# ----------------------------------------------------------------------

sub parse_configuration {
#
# Takes the input data and separates the configuration data from the
# dates/event data.
# 
    my( $data, $year, $month ) = @_;
    my( %directives, %events );

    foreach my $line ( @{$data} ) {
	if ( $line =~ m/^\w+\=/ ) {	# look for configuration lines
	    my($name, $value) = split(/=/, $line);
	    $directives{ lc($name) } = $value;
	}
	elsif ( $line =~ m/^\w+\t/ ) {
	    my( $e_year, $e_mon, $e_day, $event ) = split(/\t/, $line);
	    next unless $e_mon and $e_day and $event;	# skip partial or empty

	    # substitute real years for the the {%date=YYYY} markers
	    $event =~ s/\{%date=(\d{4})\}/$year - $1/ge;

	    # turn wildcards ('any') into actual values
	    if ( $e_year =~ m/any/i ) { $e_year = $year; }
	    if ( $e_mon =~ m/any/i ) { $e_mon = $month; }
	    if ( $e_day =~ m/any/i ) { $e_day = 'any'; }

	    $events{ join('.',$e_year,$e_mon,$e_day) } .= qq|$event<BR>\n|;

	    # interpolate the ranges for whichever years, days and months
	    if (($e_year =~ /-/) or ($e_mon =~ /-/) or ($e_day =~ /-/)) {
		foreach my $year_part ( split_range( $e_year ) ) {
		    foreach my $month_part ( split_range( $e_mon ) ) {
			foreach my $day_part ( split_range( $e_day ) ) {

			    my $date = join( '.', $year_part, $month_part,
						  $day_part );

			    $events{ $date } .= qq|$event<BR>\n|;
			}
		    }
		}
	    }
	}
    }

    return( \%directives, \%events );
}

# ----------------------------------------------------------------------

sub split_range {
#
# turns a string 'n-m' into a list containing the values n through m.
# if n and m are switched, it still tries to do the right thing and returns
# a list containing the values n through m. 
#
    my( $range ) = @_;
    my( $begin, $end ) = split(/-/, $range );

    my @list;
    if ( $end and $begin < $end ) { @list = ( $begin .. $end ); } 
    elsif ( $end and $begin > $end ) { @list = ( $end .. $begin ); }
    else { @list = $begin; }

    return @list;
}

# ----------------------------------------------------------------------

sub print_response {
#
# Sends the data to the browser (or STDOUT).  Can be called repeatedly
# as it only sends out a header the first time.
#
    my( $data, $type ) = @_;
    $type = 'text/html' unless $type;

    print qq|Content-Type: $type\n\n| unless $header_sent;
    $header_sent = 'yes';
 
    print $data;
    undef;
}

# ----------------------------------------------------------------------

